<?php
namespace FASTPIXEL;
defined('ABSPATH') || exit;
use FASTPIXEL\FASTPIXEL_Config_Model;

if (!class_exists('FASTPIXEL\FASTPIXEL_Backend_Cache')) {
    class FASTPIXEL_Backend_Cache extends FASTPIXEL_Backend_Controller
    {
        protected static $instance;
        protected $cache_dir;
        protected $config;
        //need this option to avoid 3 page cache requests
        protected $time_to_wait = 5;
        protected $time_to_wait_purge_all = 5 * 60; //5 mins till next purge all
        protected $serve_stale;
        protected $be_functions;

        function __construct()
        {
            parent::__construct();

            self::$instance = $this;
            $this->config = FASTPIXEL_Config_Model::get_instance();
            $this->cache_dir = $this->functions->get_cache_dir();
            $this->be_functions = FASTPIXEL_Backend_Functions::get_instance();


            $this->serve_stale = function_exists('get_option') ? $this->functions->get_option('fastpixel_serve_stale') : $this->config->get_option('serve_stale');

            //cache functions only for backend
            add_action('save_post', function ($post_id, $post, $update) {
                FASTPIXEL_DEBUG::log('ACTION save_post');
                if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
                    return;
                }
                if (isset($_REQUEST['fastpixel-nonce']) && !empty($_REQUEST['fastpixel-nonce']) ) {
                    if (!wp_verify_nonce(sanitize_key($_REQUEST['fastpixel-nonce']), 'fastpixel_edit_post')) {
                        return;
                    }
                }
                //need this check for admin posts/pages listing page to avoid cache request
                $action = isset($_GET['action']) && !empty($_GET['action']) ? sanitize_text_field($_GET['action']) : false;
                if ($action && $action == 'untrash') {
                    return;
                }
                if ($post->post_status == 'publish') {
                    if (!defined('FASTPIXEL_SAVE_POST')) {
                        define('FASTPIXEL_SAVE_POST', true);
                    }
                    $cache = FASTPIXEL_Backend_Cache::get_instance();
                    $cache->purge_post($post_id);
                }
            }, 10, 3);
            add_action('wp_insert_post', function ($post_id, $post, $update) {
                FASTPIXEL_DEBUG::log('ACTION wp_insert_post');
                if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
                    return;
                }
                if (isset($_REQUEST['fastpixel-nonce']) && !empty($_REQUEST['fastpixel-nonce'])) {
                    if (!wp_verify_nonce(sanitize_key($_REQUEST['fastpixel-nonce']), 'fastpixel_edit_post')) {
                        return;
                    }
                }
                //need this check for admin posts/pages listing page to avoid cache request
                $action = isset($_GET['action']) && !empty($_GET['action']) ? sanitize_text_field($_GET['action']) : false;
                if ($action && $action == 'untrash') {
                    return;
                }
                if ($post->post_status == 'publish') {
                    $cache = FASTPIXEL_Backend_Cache::get_instance();
                    $cache->purge_post($post_id);
                }
            }, 10, 3);
            add_action('draft_to_publish', function ($post) {
                FASTPIXEL_DEBUG::log('ACTION draft_to_publish');
                if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
                    return;
                }
                if (!empty($post) && $post->post_status == 'publish') {
                    $cache = FASTPIXEL_Backend_Cache::get_instance();
                    $cache->purge_post($post->ID);
                }
            }, 10, 1);
            add_action('pending_to_publish', function ($post) {
                FASTPIXEL_DEBUG::log('ACTION pending_to_publish');
                if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
                    return;
                }
                if (!empty($post)) {
                    $cache = FASTPIXEL_Backend_Cache::get_instance();
                    $cache->purge_post($post->ID);
                }
            }, 10, 1);
            add_action('transition_comment_status', function ($new_status, $old_status, $comment) {
                $cache = FASTPIXEL_Backend_Cache::get_instance();
                $cache->purge_post($comment->comment_post_ID);
            }, 10, 3);
            add_action('comment_post', function ($comment_ID, $comment_approved, $commentdata) {
                if ($comment_approved == 1) {
                    $comment = get_comment($comment_ID);
                    $cache = FASTPIXEL_Backend_Cache::get_instance();
                    $cache->purge_post($comment->comment_post_ID);
                }
            }, 10, 3);
            add_action('admin_post_fastpixel_admin_purge_post_cache', [$this, 'admin_purge_post_cache']);
            add_action('admin_post_fastpixel_admin_purge_homepage_cache', [$this, 'admin_purge_homepage_cache']);
            add_action('admin_post_fastpixel_admin_delete_cached', [$this, 'admin_delete_cached_files']);
            //taxonomy functions
            add_action('edit_terms', function ($term_id, $taxonomy, $args) {
                FASTPIXEL_DEBUG::log('ACTION edit_terms');
                $current_term = get_term($term_id);
                if ($current_term->slug != $args['slug']) {
                    $url = new FASTPIXEL_Url(null, get_term_link($current_term));
                    $this->delete_url_cache($url);
                }}, 10, 3);
            add_action('delete_term', function ($term_id, $taxonomy, $args, $deleted_term) {
                FASTPIXEL_DEBUG::log('ACTION delete_term');
                if (!empty($deleted_term)) {
                    $url = new FASTPIXEL_Url(null, get_term_link($deleted_term));
                    $this->delete_url_cache($url);
                }
            }, 11, 4);
            add_action('edited_terms', function ($term_id, $taxonomy, $args) {
                FASTPIXEL_DEBUG::log('ACTION edited_terms');
                $term = get_term($term_id);
                //according docs there should be already updated term, but it is not, in this case we update slug manually
                if ($args['slug'] != $term->slug) {
                    $term->slug = $args['slug'];
                }
                $url = new FASTPIXEL_Url(null, get_term_link($term));
                $this->functions->update_post_cache($url->get_url_path(), true);
            }, 10, 3);

            add_action('permalink_structure_changed', [$this, 'permalinks_change'], 10, 2);

            //moved this action outside is_admin validation to get old terms that needs to be reset, because is_admin fires after 'pre_post_update'
            //this is required in case gutenberg editor is used
            add_action('pre_post_update', function ($post_id, $data) {
                FASTPIXEL_Debug::log('ACTION pre_post_update');
                //need this to avoid duplicate execution for post type "post" which use gutenberg
                //TODO: more accurate validation
                if ($data['post_type'] == 'post' && function_exists('use_block_editor_for_post') && !use_block_editor_for_post($post_id)) {
                    return;
                }
                $post = get_post($post_id);
                //do actions when post status changed
                if ($post->post_status != $data['post_status']) {
                    if ($post->post_status == 'publish' && $data['post_status'] != 'trash') {
                        //resetting/removing cache when post is "unpublished"
                        $url = new FASTPIXEL_Url($post_id);
                        $this->delete_url_cache($url);
                    }
                    //if post was published and still have publish status then we need to check slug, it can be changed too
                    if ($post->post_status == 'publish' && $post->post_status == $data['post_status'] && $post->post_name != $data['post_name']) {
                        $url = new FASTPIXEL_Url($post_id);
                        $this->delete_url_cache($url);
                    }
                    //trashed posts served separately because their name contains word "__trashed" already
                    if ($data['post_status'] == 'trash' && $post->post_status == 'publish') {
                        $post->post_name = preg_replace('/__trashed/i', '', $post->post_name);
                        $url = new FASTPIXEL_Url(null, get_the_permalink($post));
                        $this->delete_url_cache($url);
                    }
                    //resetting archives and taxonomies pages
                    $this->admin_purge_archives($post_id);
                    $this->admin_purge_taxonomies($post_id);
                }
                //need to check if post slug changed
                if ($post->post_name != $data['post_name']) {
                    //need to delete old cached slug
                    FASTPIXEL_Debug::log('PRE_POST_UPDATE: Slug changed. Old post slug that should be removed', get_the_permalink($post));
                    $url = new FASTPIXEL_Url(null, get_the_permalink($post));
                    $this->delete_url_cache($url);
                }
                //need to check if post parent changed
                if ($post->post_parent != $data['post_parent']) {
                    //need to delete old cached slug
                    FASTPIXEL_Debug::log('PRE_POST_UPDATE: Parent changed. Old post slug that should be removed', get_the_permalink($post));
                    $url = new FASTPIXEL_Url(null, get_the_permalink($post));
                    $this->delete_url_cache($url);
                }
            }, 10, 2);

            add_action('wp_ajax_fastpixel_purge_post_cache', [$this, 'admin_ajax_purge_post_cache']);
            add_action('wp_ajax_fastpixel_cache_statuses', [$this, 'admin_ajax_cache_statuses']);
            add_action('wp_ajax_fastpixel_delete_cached_files', [$this, 'admin_ajax_delete_cached_files']);

            /*
             * added nonce to the edit form, not works with gutenberg editor 
             */
            add_action('edit_page_form', [$this, 'post_nonce_field']);
            add_action('edit_form_advanced', [$this, 'post_nonce_field']);
        }

        public static function get_instance()
        {
            if (empty(self::$instance)) {
                self::$instance = new FASTPIXEL_Backend_Cache();
            }
            return self::$instance;
        }

        public function post_nonce_field() {
            wp_nonce_field('fastpixel_edit_post', 'fastpixel-nonce', false);
        }

        public function purge_post($post_id)
        {
            if (empty($post_id)) {
                return false;
            }
            $post = get_post(esc_sql($post_id));

            // Do not purge the cache if it's an autosave or it is updating a revision.
            if ((defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) || 'revision' === $post->post_type) {
                return false;

            // Do not purge the cache if the user is editing an unpublished post.
            } elseif (in_array($post->post_status, ['draft', 'auto-draft', 'pending'])) {
                return false;

            // Do not purge the cache if the user cannot edit the post.
            } elseif (!current_user_can('edit_post', $post_id) && (!defined('DOING_CRON') || !DOING_CRON)) {
                return false;
            }

            //Do not purge post if post type is excluded
            $excluded_post_types = apply_filters('fastpixel_purge_excluded_post_types', []);
            $post_type = get_post_type($post_id);
            if (in_array($post_type, $excluded_post_types)) {
                return false;
            }

            $url = new FASTPIXEL_Url($post_id);
            $path = untrailingslashit($url->get_url_path());
            //need to check if request already was sent, because on post save much hooks are fired and each can trigger cache request
            $status = $this->functions->check_post_cache_status($url->get_url());
            if (time() <= ($this->time_to_wait + $status['last_cache_request_time'])) {
                return false;
            }
            //delete cached files if serve stale is disabled(adding this to keep more free space and avoid serving cached pages)
            if (!$this->functions->get_option('fastpixel_serve_stale')) {
                $this->functions->delete_cached_files($path);
            }
            $this->functions->update_post_cache($path, true);
            //request page cache after reset
            if (class_exists('FASTPIXEL\FASTPIXEL_Cache') && $cache = FASTPIXEL_Cache::get_instance()) {
                if ($cache->request_page_cache($post_id)) {
                    //purging categories/tags/taxonomies pages
                    $this->admin_purge_taxonomies($post_id);
                    return true;
                } else {
                    return false;
                }
            } else {
                FASTPIXEL_DEBUG::log('Class FASTPIXEL_CACHE is not available during post purge');
            }
            return true;
        }

        function purge_all()
        {
            //initializing filesystem
            global $wp_filesystem;
            if (empty($wp_filesystem)) {
                require_once(ABSPATH . '/wp-admin/includes/file.php');
                WP_Filesystem();
            }
            if (file_exists($this->functions->get_cache_dir())) {
                $wp_filesystem->put_contents($this->functions->get_cache_dir() . DIRECTORY_SEPARATOR . 'invalidated', '');
            } else {
                FASTPIXEL_Debug::log('Can\'t purge cache, cache directory not exists');
                return false;
            }
            $do_request = apply_filters('fastpixel_request_purge_all', true);
            FASTPIXEL_DEBUG::log('Purge All $do_request', $do_request);
            //request page cache after reset
            if ($do_request) {
                if (class_exists('FASTPIXEL\FASTPIXEL_Cache') && $cache = FASTPIXEL_Cache::get_instance()) {
                    $api_url = FASTPIXEL_API_HOST.'/api/v1/purge_all';
                    FASTPIXEL_DEBUG::log("Requesting Purge All API", $api_url);
                    if (is_multisite()) {
                        $purge_url = network_home_url();
                    } else {
                        $purge_url = home_url();
                    }
                    $cache->api_request($api_url, ['url' => $purge_url]);
                } else {
                    FASTPIXEL_DEBUG::log('Class FASTPIXEL_CACHE is not available in "purge all" action');
                }
            }
            if (function_exists('wp_cache_flush')) {
                wp_cache_flush();
            }
            do_action('fastpixel_purge_all');
            return true;
        }

        function admin_purge_post_cache() 
        {
            if (!isset($_GET['nonce']) || !wp_verify_nonce(sanitize_text_field($_GET['nonce']), 'cache_status_nonce')) {
                $this->notices->add_flash_notice(esc_html__('Can\'t run action, wrong nonce provided', 'fastpixel-website-accelerator'), 'error', false);
                wp_redirect(wp_get_referer());
                return;
            }
            $post_id = sanitize_text_field($_GET['post_id']);
            if (empty($post_id) || !is_numeric($post_id)) {
                return;
            }
            $post = get_post($post_id);
            $status = $this->functions->check_post_cache_status(get_the_permalink($post));
            if ($this->purge_post($post_id)) {  
                $status_text = $status['have_cache'] && !$status['need_cache'] ?
                    /* translators: status purged */
                    esc_html__('purged.', 'fastpixel-website-accelerator') :
                    /* translators: status requested */
                    esc_html__('requested.', 'fastpixel-website-accelerator');
                /* translators: %1 post name, %2 action name (purged or requested) */
                $this->notices->add_flash_notice(sprintf(esc_html__('Cache for post "%1$s" has been %2$s', 'fastpixel-website-accelerator'),  esc_html($post->post_title), esc_html($status_text)), 'success');
            }
            wp_redirect(wp_get_referer());
        }

        public function admin_ajax_purge_post_cache()
        {
            if (!isset($_POST['nonce']) || !wp_verify_nonce(sanitize_text_field($_POST['nonce']), 'cache_status_nonce')) {
                echo wp_json_encode(['status' => 'error', 'statusText' => esc_html__('Can\'t run action, wrong nonce provided', 'fastpixel-website-accelerator')]);
                wp_die();
            }
            $post_id = sanitize_text_field($_POST['post_id']);
            if (empty($post_id)) {
                echo wp_json_encode(['status' => 'error', 'statusText' => esc_html__('Post ID is wrong or empty', 'fastpixel-website-accelerator')]);
                wp_die();
            }
            //initializing filesystem
            global $wp_filesystem;
            if (empty($wp_filesystem)) {
                require_once(ABSPATH . '/wp-admin/includes/file.php');
                WP_Filesystem();
            }
            if (is_numeric($post_id)) {
                $post = get_post($post_id);
                $post_permalink = get_the_permalink($post);
                $status = $this->functions->check_post_cache_status($post_permalink);
                if ($this->purge_post($post_id)) {
                    $status_text = $status['have_cache'] && !$status['need_cache'] ?
                        /* translators: status purged */
                        esc_html__('purged.', 'fastpixel-website-accelerator') :
                        /* translators: status requested */
                        esc_html__('requested.', 'fastpixel-website-accelerator');
                    /* translators: %1 post name, %2 action name (purged or requested) */
                    $post_status = $this->be_functions->cache_status_display($post_permalink);
                    echo wp_json_encode([
                        'post_id'    => $post_id,
                        'status'     => 'success',
                        /* translators: %1 used to display post name, %2 should display text "purged" or "requested"(texts are translated separately) */
                        'statusText' => sprintf(esc_html__('Cache for post "%1$s" has been %2$s', 'fastpixel-website-accelerator'), esc_html($post->post_title), esc_html($status_text)),
                        'post'       => $post_status
                    ]);
                    wp_die();
                }
            } else if ($post_id == 'homepage') {
                $url = new FASTPIXEL_Url(null, get_home_url());
                $path = untrailingslashit($url->get_url_path());
                $this->functions->update_post_cache($path, true);
                //request page cache after reset
                if (class_exists('FASTPIXEL\FASTPIXEL_Cache') && $cache = FASTPIXEL_Cache::get_instance()) {
                    $meta_path = $this->cache_dir . DIRECTORY_SEPARATOR . $path . DIRECTORY_SEPARATOR . 'meta';
                    //checking for meta file
                    if (file_exists($meta_path)) {
                        $meta = json_decode($wp_filesystem->get_contents($meta_path), true);
                    }
                    //check for last request time and prevent new request in case 
                    if (
                        isset($meta) &&
                        (isset($meta['invalidated_time']) && $meta['invalidated_time'] < $meta['cache_request_time']) &&
                        (isset($meta['cache_request_time']) && time() < $meta['cache_request_time'] + $cache->get_request_wait_time())
                    ) {
                        //TODO: check if this message is required
                        /* translators: %1 seconds count */
                        echo wp_json_encode(['status' => 'error', 'statusText' => sprintf(esc_html__('The cache for "Homepage" has been cleared and is not yet ready. Must wait %1$d seconds to try again.', 'fastpixel-website-accelerator'), esc_html($cache->get_request_wait_time()))]);
                        wp_die();
                    }
                    $status = $this->functions->check_post_cache_status(get_home_url());
                    //Doing Page Cache request to Remote Service Server
                    if ($cache->cache_request($url->get_url())) {
                        $this->functions->update_post_cache($path, false, true);
                        $status_text = $status['have_cache'] && !$status['need_cache'] ?
                            /* translators: status purged */
                            esc_html__('purged.', 'fastpixel-website-accelerator') :
                            /* translators: status requested */
                            esc_html__('requested.', 'fastpixel-website-accelerator');
                        /* translators: %1 post name, %2 action name (purged or requested) */
                        $post_status = $this->be_functions->cache_status_display(get_home_url());
                        echo wp_json_encode([
                            'post_id'    => $post_id,
                            'status'     => 'success',
                            /* translators: %s should display text "purged" or "requested"(texts are translated separately) */
                            'statusText' => sprintf(esc_html__('Cache for "Homepage" has been %s', 'fastpixel-website-accelerator'), esc_html($status_text)),
                            'post'       => $post_status
                        ]);
                        wp_die();
                    }
                }
            }
            echo wp_json_encode(['status' => 'error', 'statusText' => esc_html__('Post ID is wrong or empty', 'fastpixel-website-accelerator')]);
            wp_die();
        }

        public function admin_ajax_cache_statuses() {
            if (!isset($_POST['nonce']) || !wp_verify_nonce(sanitize_text_field($_POST['nonce']), 'cache_status_nonce')) {
                echo wp_json_encode(['status' => 'error', 'statusText' => esc_html__('Can\'t run action, wrong nonce provided', 'fastpixel-website-accelerator')]);
                wp_die();
            }
            $post_ids = isset($_POST['post_ids']) ? (array)$_POST['post_ids'] : [];
            if (empty($post_ids)) {
                echo wp_json_encode(['status' => 'error', 'statusText' => esc_html__('Post IDs wrong or empty', 'fastpixel-website-accelerator')]);
                wp_die();
            }
            $statuses = ['status' => 'success', 'posts' => []];
            foreach ($post_ids as $post_id) {
                if ($post_id == 'homepage') {
                    $permalink = get_home_url();
                } else {
                    $permalink = get_permalink($post_id);
                }
                $status = $this->be_functions->cache_status_display($permalink);
                if ($status) {
                    $statuses['posts'][$post_id] = $status;
                }
            }

            echo wp_json_encode($statuses);
            wp_die();
        }

        public function admin_ajax_delete_cached_files()
        {
            if (!isset($_POST['nonce']) || !wp_verify_nonce(sanitize_text_field($_POST['nonce']), 'cache_status_nonce')) {
                echo wp_json_encode(['status' => 'error', 'statusText' => esc_html__('Can\'t run action, wrong nonce provided', 'fastpixel-website-accelerator')]);
                wp_die();
            }
            $post_id = sanitize_text_field($_POST['post_id']);
            if (empty($post_id)) {
                echo wp_json_encode(['status' => 'error', 'statusText' => esc_html__('Post IDs wrong or empty', 'fastpixel-website-accelerator')]);
                wp_die();
            }
            if ($post_id == 'homepage') {
                $permalink = get_home_url();
            } else {
                $permalink = get_permalink($post_id);
            }
            $url = new FASTPIXEL_Url(null, $permalink);
            if ($this->delete_url_cache($url)) {
                $post_status = $this->be_functions->cache_status_display($url->get_url());
                echo wp_json_encode([
                    'post_id'    => $post_id,
                    'status'     => 'success',
                    /* translators: %s used to display post name */
                    'statusText' => sprintf(esc_html__('Cached files deleted for %s.', 'fastpixel-website-accelerator'), esc_html($url->get_url())),
                    'post'       => $post_status
                ]);
            } else {
                echo wp_json_encode(['status' => 'error', 'statusText' => esc_html__('Error occured while deleting', 'fastpixel-website-accelerator')]);
            }
            wp_die();
        }

        //function that have duplicate functionality for cache request(need this only when static page is not set)
        public function admin_purge_homepage_cache()
        {
            if (!isset($_GET['nonce']) || !wp_verify_nonce(sanitize_text_field($_GET['nonce']), 'cache_status_nonce')) {
                $this->notices->add_flash_notice(esc_html__('Can\'t run action, wrong nonce provided', 'fastpixel-website-accelerator'), 'error', false);
                wp_redirect(wp_get_referer());
                return;
            }
            //initializing filesystem
            global $wp_filesystem;
            if (empty($wp_filesystem)) {
                require_once(ABSPATH . '/wp-admin/includes/file.php');
                WP_Filesystem();
            }
            $url = new FASTPIXEL_Url(null, get_home_url());
            $path = untrailingslashit($url->get_url_path());
            //need to check if request already was sent, because on post save much hooks are fired and each can trigger cache request
            $status = $this->functions->check_post_cache_status($url->get_url());
            if (time() <= ($this->time_to_wait + $status['last_cache_request_time'])) {
                wp_redirect(wp_get_referer());
            }
            $this->functions->update_post_cache($path, true);
            //request page cache after reset
            if (class_exists('FASTPIXEL\FASTPIXEL_Cache') && $cache = FASTPIXEL_Cache::get_instance()) {
                $meta_path = $this->cache_dir . DIRECTORY_SEPARATOR . $path . DIRECTORY_SEPARATOR . 'meta';
                //checking for meta file
                if (file_exists($meta_path)) {
                    $meta = json_decode($wp_filesystem->get_contents($meta_path), true);
                }
                //check for last request time and prevent new request in case 
                if (
                    isset($meta) &&
                    (isset($meta['invalidated_time']) && $meta['invalidated_time'] < $meta['cache_request_time']) &&
                    (isset($meta['cache_request_time']) && time() < $meta['cache_request_time'] + $cache->get_request_wait_time())
                ) {
                    //TODO: check if this message is required
                    /* translators: %1 seconds count */
                    $this->notices->add_flash_notice(sprintf(__('The cache for "Homepage" has been cleared and is not yet ready. Must wait %1$d seconds to try again.', 'fastpixel-website-accelerator'), esc_html($cache->get_request_wait_time())));
                    return;
                }
                //Doing Page Cache request to Remote Service Server
                $cache->cache_request($url->get_url());
                $this->functions->update_post_cache($path, false, true);
                $this->notices->add_flash_notice(esc_html__('Cache for "Homepage" has been purged.', 'fastpixel-website-accelerator'), 'success');
            } else {
                FASTPIXEL_DEBUG::log('Class FASTPIXEL_CACHE is not available during homepage cache purge');
            }
            wp_redirect(wp_get_referer());
        }

        function admin_purge_archives($post_id = null) {
            if (empty($post_id) || !is_numeric($post_id)) {
                return;
            }
            $post_type = get_post_type($post_id);

            // test case - default wordpress installation
            if ($post_type === 'post') {
                if (get_option('show_on_front') === 'posts') {
                    $path = untrailingslashit(preg_replace('/https?:\/\//i', '', home_url()));
                    $this->functions->update_post_cache($path, true);
                } else {
                    // if blog page is set then it needs to be reset
                    $blog_page_id = get_option('page_for_posts');
                    $blog_url = new FASTPIXEL_Url($blog_page_id);
                    $this->functions->update_post_cache($blog_url->get_url_path(), true);
                }
            } else {
                $post_types = array_keys(get_post_types(['public' => true, 'has_archive' => true], 'object'));
                if ($post_type && in_array($post_type, $post_types)) {
                    $archive_url = get_post_type_archive_link($post_type);
                    if ($archive_url) {
                        $url = new FASTPIXEL_Url(null, $archive_url);
                        $path = untrailingslashit($url->get_url_path());
                        $this->functions->update_post_cache($path, true);
                    }
                }
            }
        }
        function admin_purge_taxonomies($post_id = null)
        {
            if (empty($post_id) || !is_numeric($post_id)) {
                return;
            }
            $taxonomies = get_post_taxonomies($post_id);
            if (!empty($taxonomies)) {
                foreach ($taxonomies as $taxonomy_name) {
                    $taxonomy = get_taxonomy($taxonomy_name);
                    if ( $taxonomy->public ||  $taxonomy->publicly_queryable) {
                        $terms = get_the_terms($post_id, $taxonomy_name);
                        if (!empty($terms)) {
                            foreach ($terms as $term) {
                                $term_link = get_term_link($term);
                                if (!empty($term_link)) {
                                    $url = new FASTPIXEL_Url(null, $term_link);
                                    $path = untrailingslashit($url->get_url_path());
                                    $this->functions->update_post_cache($path, true);
                                }
                            }
                        }
                    }
                }
            }
        }

        //function that delete cached files and folders when page is unpublished, deleted, renamed etc... 
        function delete_url_cache($url = null) {
            if (empty($url) || !is_object($url)) {
                return false;
            }
            return $this->functions->delete_cached_files($url->get_url_path());
        }

        //update advanced-cache.php on rewrite rules change
        function permalinks_change($old, $new)
        {
            $diag = FASTPIXEL_Diag::get_instance();
            //checking same activation test like on plugin activation
            if ($diag->run_activation_tests()) {
                $this->functions->update_ac_file();
            }
            return $new;
        }

        function admin_delete_cached_files() {
            if (!isset($_GET['nonce']) || !wp_verify_nonce(sanitize_text_field($_GET['nonce']), 'cache_status_nonce')) {
                $this->notices->add_flash_notice(esc_html__('Can\'t run action, wrong nonce provided', 'fastpixel-website-accelerator'), 'error', false);
                wp_redirect(wp_get_referer());
                return;
            }
            $post_id = isset($_GET['post_id']) ? sanitize_text_field($_GET['post_id']) : false;
            if (empty($post_id)) {
                $this->notices->add_flash_notice(esc_html__('Cache files cannot be deleted, post_id does not exist.', 'fastpixel-website-accelerator'), 'error', false);
                wp_redirect(wp_get_referer());
            }
            if ($post_id == 'homepage') {
                $url = get_site_url();
            } else {
                $url = get_the_permalink($post_id);
            }
            $url = new FASTPIXEL_Url(null, $url);
            if ($this->delete_url_cache($url)) {
                /* translators: %1 should be a url*/
                $this->notices->add_flash_notice(sprintf(esc_html__('Cached files deleted for %1$s', 'fastpixel-website-accelerator'), esc_url($url->get_url())), 'success', false);
            }
            wp_redirect(wp_get_referer());
        }
    }

    new FASTPIXEL_Backend_Cache();
}