<?php
namespace FASTPIXEL;
defined('ABSPATH') || exit;

if (!class_exists('FASTPIXEL\FASTPIXEL_Backend_Functions')) {
    class FASTPIXEL_Backend_Functions
    {
        public static $instance;
        protected $functions;
        protected $excludes;
        protected $serve_stale;

        public function __construct()
        {
            self::$instance = $this;
            $this->functions = FASTPIXEL_Functions::get_instance();
            $this->excludes = FASTPIXEL_Excludes::get_instance();
            $this->serve_stale = $this->functions->get_option('fastpixel_serve_stale');
        }

        public static function get_instance()
        {
            if (empty(self::$instance)) {
                self::$instance = new FASTPIXEL_Notices();
            }
            return self::$instance;
        }

        public function cache_status_display($url)
        {
            $cache_status = [
                'status' => '',
                'status_display' => ''
            ];
            $check_result = $this->functions->check_post_cache_status($url);
            $cache_status['html_created_time'] = $check_result['html_created_time'];
            //need to check for exclusion first
            $is_exclusion = $this->excludes->check_is_exclusion($url);
            if ($is_exclusion) {
                $cache_status['status_display'] = '<span class="have-popup"><strong>' . esc_html__('Excluded', 'fastpixel-website-accelerator') . '</strong></span>';
                $cache_status['status_display'] .= '<span class="have-popup dashicons dashicons-editor-help"></span>';
                $cache_status['status_display'] .= '<div class="pop-up">' . esc_html__('URL is excluded or has dynamic content.', 'fastpixel-website-accelerator') . '</div>';
                $cache_status['status'] = 'excluded';
                return $cache_status;
            }

            //checking page status
            if ($check_result['error'] != false && $check_result['error_time'] > $check_result['last_cache_request_time']) {
                $cache_status['status_display'] = '<span class="have-popup error"><strong>' . esc_html__('Error', 'fastpixel-website-accelerator') . '</strong></span>';
                $cache_status['status'] = 'error';
                /* translators: %s should be an error text */
                $cache_status['status_display'] .= '<div class="pop-up">' . sprintf(esc_html__('Error: %s', 'fastpixel-website-accelerator'), $check_result['error']) . '</div>';
            } else {
                if ($check_result['have_cache'] && !$check_result['need_cache']) {
                    $cache_status['status_display'] = '<span class="have-popup cached"><strong>' . esc_html__('Cached', 'fastpixel-website-accelerator') . '</strong></span>';
                    $cache_status['status'] = 'cached';
                    /* translators: %s should be a date string */
                    $cache_status['status_display'] .= '<div class="pop-up">' . sprintf(esc_html__('Cached at: %s', 'fastpixel-website-accelerator'), gmdate('Y-m-d H:i:s', $check_result['html_created_time'])) . '</div>';
                } else if ($check_result['have_cache'] && $check_result['need_cache'] && $this->serve_stale) {
                    $cache_status['status_display'] = '<div class="stale-container"><span class="have-popup cached invalidated"><strong>' . esc_html__('Stale', 'fastpixel-website-accelerator') . '</strong></span><span class="stale-loader"></span></div>';
                    $cache_status['status'] = 'stale';
                    /* translators: %s should be a date string */
                    $cache_status['status_display'] .= '<div class="pop-up">' . sprintf(esc_html__('Cached at: %s', 'fastpixel-website-accelerator'), gmdate('Y-m-d H:i:s', $check_result['html_created_time'])) . '<br/>' .
                        /* translators: %s should be a date string */
                        sprintf(esc_html__('Cache requested at: %s', 'fastpixel-website-accelerator'), gmdate('Y-m-d H:i:s', $check_result['last_cache_request_time'])) . '</div>';
                } else {
                    if ($check_result['last_cache_request_time'] && ($check_result['have_cache'] == false || $check_result['last_cache_request_time'] > $check_result['html_created_time'])) {
                        $cache_status['status_display'] = '<div class="queued-container"><span class="have-popup queued">' . esc_html__('Queued', 'fastpixel-website-accelerator') . '</span><span class="queued-loader"></span></div>';
                        $cache_status['status'] = 'queued';
                        /* translators: %s should be a date string */
                        $cache_status['status_display'] .= '<div class="pop-up">' . 
                            /* translators: %s is used to display date and time when page cache was requested */
                            sprintf(esc_html__('Cache requested at: %s', 'fastpixel-website-accelerator'), gmdate('Y-m-d H:i:s', $check_result['last_cache_request_time'])) . 
                            /* translators: */
                            ( $check_result['error'] != false ? '<br/>' . sprintf(esc_html__('Last Error: %s', 'fastpixel-website-accelerator'), $check_result['error']) : '' ) .
                        '</div>';
                    } else {
                        $cache_status['status_display'] = '<span class="not-cached">' . esc_html__('Not Cached', 'fastpixel-website-accelerator') . '</span>';
                        $cache_status['status_display'] .= '<span class="have-popup dashicons dashicons-editor-help"></span>';
                        $cache_status['status_display'] .= '<div class="pop-up">' . esc_html__('To efficiently use the resources on your website, pages are processed and cached as they are visited by outside visitors. You can manually force a page to be processed by clicking "Cache now"', 'fastpixel-website-accelerator') . '</div>';
                        $cache_status['status'] = 'not-cached';
                    }
                }
            }
            return $cache_status;
        }
    }

    new FASTPIXEL_Backend_Functions();
}