<?php
namespace FASTPIXEL;
defined('ABSPATH') || exit;

if (!class_exists('FASTPIXEL\FASTPIXEL_Tab_Javascript')) {
    class FASTPIXEL_Tab_Javascript extends FASTPIXEL_UI_Tab
    {

        protected $slug = 'javascript';
        protected $order = 4;

        public function __construct()
        {
            parent::__construct();
            $this->name = esc_html__('JavaScript', 'fastpixel-website-accelerator');
            add_filter('sanitize_option_fastpixel_javascript_optimization', [$this, 'sanitize_fastpixel_javascript_optimization_cb'], 10, 3);
            $this->save_options();
        }

        function settings()
        {
            register_setting(FASTPIXEL_TEXTDOMAIN, 'fastpixel_javascript_optimization', ['type' => 'integer']);
            register_setting(FASTPIXEL_TEXTDOMAIN, 'fastpixel_javascript_excludes', ['type' => 'array']);
            register_setting(FASTPIXEL_TEXTDOMAIN, 'fastpixel_javascript_excludes_regexp', ['type' => 'array']);
            register_setting(FASTPIXEL_TEXTDOMAIN, 'fastpixel_javascript_exclude_gdpr', ['type' => 'boolean']);
            add_settings_section(
                'fastpixel_settings_section-javascript',
                '',
                false,
                FASTPIXEL_TEXTDOMAIN . '-javascript'
            );
            add_settings_field(
                'fastpixel_javascript_optimization',
                esc_html__('Optimization', 'fastpixel-website-accelerator'),
                [$this, 'field_fastpixel_javascript_optimization_cb'],
                FASTPIXEL_TEXTDOMAIN . '-javascript',
                'fastpixel_settings_section-javascript'
            );
            add_settings_field(
                'fastpixel_javascript_excludes',
                esc_html__('Exclusions', 'fastpixel-website-accelerator'),
                [$this, 'field_fastpixel_javascript_excludes_cb'],
                FASTPIXEL_TEXTDOMAIN . '-javascript',
                'fastpixel_settings_section-javascript'
            );
            add_settings_field(
                'fastpixel_javascript_excludes_regexp',
                esc_html__('RegExp Exclusions', 'fastpixel-website-accelerator'),
                [$this, 'field_fastpixel_javascript_excludes_regexp_cb'],
                FASTPIXEL_TEXTDOMAIN . '-javascript',
                'fastpixel_settings_section-javascript'
            );
            add_settings_field(
                'fastpixel_javascript_exclude_gdpr',
                esc_html__('GDPR', 'fastpixel-website-accelerator'),
                [$this, 'field_fastpixel_javascript_exclude_gdpr_cb'],
                FASTPIXEL_TEXTDOMAIN . '-javascript',
                'fastpixel_settings_section-javascript'
            );
        }
        public function sanitize_fastpixel_javascript_optimization_cb($value, $option, $original_value) 
        {
            $old_value = $this->functions->get_option($option);
            if ($value != $old_value && !defined('FASTPIXEL_PURGE_ON_IMAGE')) {
                define('FASTPIXEL_PURGE_ON_JS', true);
                $backend_cache = FASTPIXEL_Backend_Cache::get_instance();
                if ($backend_cache->purge_all()) {
                    $notices = FASTPIXEL_Notices::get_instance();
                    $notices->add_flash_notice(esc_html__('Javascript optimization changed. Cache cleared!', 'fastpixel-website-accelerator'), 'success', true);
                }
            }
            return $value;
        }
        public function field_fastpixel_javascript_optimization_cb($args)
        {
            // Get the value of the setting we've registered with register_setting()
            $option = $this->functions->get_option('fastpixel_javascript_optimization');
            ?>
            <div class="fastpixel-select-with-description fastpixel-select">
                <div class="fastpixel-row">
                    <select id="fastpixel_javascript_optimization" name="fastpixel_javascript_optimization">
                        <option value="1" <?php echo $option == 1 ? 'selected="selected"' : ''; ?>><?php esc_html_e('Optimize JavaScript', 'fastpixel-website-accelerator'); ?></option>
                        <option value="2" <?php echo $option == 2 ? 'selected="selected"' : ''; ?>><?php esc_html_e('Delay non-critical JavaScript', 'fastpixel-website-accelerator'); ?></option>
                        <option value="3" <?php echo $option == 3 ? 'selected="selected"' : ''; ?>><?php esc_html_e('Do not optimize JavaScript', 'fastpixel-website-accelerator'); ?></option>
                    </select>
                    <div class="field-description">
                        <span class="optimization-description fastpixel-desc-hidden" data-value="1"><?php esc_html_e('All scripts are optimized and run as on the original page.', 'fastpixel-website-accelerator'); ?></span>
                        <span class="optimization-description fastpixel-desc-hidden" data-value="2"><?php esc_html_e('All scripts are optimized and delayed, except necessary scripts like GDPR.', 'fastpixel-website-accelerator'); ?></span>
                        <span class="optimization-description fastpixel-desc-hidden" data-value="3"><?php esc_html_e('No JavaScript optimization is performed, the scripts run exactly as on the original page.', 'fastpixel-website-accelerator'); ?></span>
                    </div>
                </div>
            </div>
            <?php
        }
        public function field_fastpixel_javascript_excludes_cb($args)
        {
            // Get the value of the setting we've registered with register_setting()
            $excludes = $this->functions->get_option('fastpixel_javascript_excludes');
            ?>
            <textarea id="fastpixel_javascript_excludes" name="fastpixel_javascript_excludes" rows="7" cols="100" data-depends-on="fastpixel-javascript-optimization" class="fastpixel-textarea"><?php echo esc_textarea($excludes); ?></textarea>
            <span class="fastpixel-field-desc fastpixel-textarea-desc"><?php esc_html_e('Scripts that should be excluded from caching. Each script URL should be added on a new line.', 'fastpixel-website-accelerator'); ?></span>
            <span class="fastpixel-field-desc fastpixel-textarea-desc"><?php esc_html_e('Example:', 'fastpixel-website-accelerator');?>&nbsp;<?php echo esc_url(home_url('/wp-includes/js/jquery/jquery.js')); ?></span>
            <?php
        }
        public function field_fastpixel_javascript_excludes_regexp_cb($args)
        {
            // Get the value of the setting we've registered with register_setting()
            $excludes = $this->functions->get_option('fastpixel_javascript_excludes_regexp');
            ?>
            <textarea id="fastpixel_javascript_excludes_regexp" name="fastpixel_javascript_excludes_regexp" rows="7" cols="100" data-depends-on="fastpixel-javascript-optimization" class="fastpixel-textarea"><?php echo esc_textarea($excludes); ?></textarea>
            <span class="fastpixel-field-desc fastpixel-textarea-desc"><?php
            /* translators: %s used to display "br" tag, nothing to translate */
            echo sprintf(esc_html__('Specify URLs, keywords or regular expressions that can identify inline or src of JavaScript to exclude from deferred execution (one per line).%s
            This is a good place to put your menus, carousels in the hero area, GA and GTM.', 'fastpixel-website-accelerator'), '<br/><br/>'); ?></span>
            <span class="fastpixel-field-desc fastpixel-textarea-desc"><?php esc_html_e('Example:', 'fastpixel-website-accelerator'); ?>&nbsp;<?php echo esc_html('fastpixel(\.min)?\.js'); ?></span>
            <?php
        }
        public function field_fastpixel_javascript_exclude_gdpr_cb($args)
        {
            // Get the value of the setting we've registered with register_setting()
            $exclude = $this->functions->get_option('fastpixel_javascript_exclude_gdpr');
            ?>
            <input type="checkbox" id="fastpixel_javascript_exclude_gdpr" name="fastpixel_javascript_exclude_gdpr" value="1" <?php echo checked($exclude); ?> data-depends-on="fastpixel-javascript-optimization" />
            <span class="fastpixel-field-desc"><?php esc_html_e('Exclude GDPR/Cookie popups', 'fastpixel-website-accelerator'); ?></span>
            <?php
        }

        public function save_options() {
            if ($_SERVER['REQUEST_METHOD'] !== 'POST' || (defined('DOING_AJAX') && DOING_AJAX) || 
                check_admin_referer('fastpixel-settings', 'fastpixel-nonce') == false ||
                empty($_POST['fastpixel-action']) || sanitize_key($_POST['fastpixel-action']) != 'save_settings') {
                return;
            }
            if (isset($_POST['fastpixel_javascript_optimization']) && is_numeric($_POST['fastpixel_javascript_optimization'])) {
                $this->functions->update_option('fastpixel_javascript_optimization', (int)sanitize_text_field($_POST['fastpixel_javascript_optimization']));
            }
            if (isset($_POST['fastpixel_javascript_excludes'])) {
                $this->functions->update_option('fastpixel_javascript_excludes', sanitize_textarea_field($_POST['fastpixel_javascript_excludes']));
            }
            if (isset($_POST['fastpixel_javascript_excludes_regexp'])) {
                $this->functions->update_option('fastpixel_javascript_excludes_regexp', sanitize_textarea_field($_POST['fastpixel_javascript_excludes_regexp']));
            }
            $gdpr = isset($_POST['fastpixel_javascript_exclude_gdpr']) && 1 == sanitize_text_field($_POST['fastpixel_javascript_exclude_gdpr']) ? 1 : 0;
            $this->functions->update_option('fastpixel_javascript_exclude_gdpr', $gdpr);    
        }
    }
    new FASTPIXEL_Tab_Javascript;
}