<?php
namespace FASTPIXEL;

defined('ABSPATH') || exit;

if (!class_exists('FASTPIXEL\FASTPIXEL_Tab_Settings')) {
    class FASTPIXEL_Tab_Settings extends FASTPIXEL_UI_Tab
    {
        protected $name = 'Settings';
        protected $slug = 'settings';
        protected $order = 3;
        public $cache_dir;
        protected $config_file;
        protected $api_key;

        public function __construct()
        {
            parent::__construct();
            $this->name = esc_html__('Settings', 'fastpixel-website-accelerator');
            $this->save_options();
        }

        function settings()
        {
            if (!$this->check_capabilities()) {
                return;
            }

            // Register a new setting for "settings" page.
            register_setting(FASTPIXEL_TEXTDOMAIN, 'fastpixel_serve_stale', ['type' => 'boolean']);
            register_setting(FASTPIXEL_TEXTDOMAIN, 'fastpixel_display_cached_for_logged', ['type' => 'boolean']);
            register_setting(FASTPIXEL_TEXTDOMAIN, 'fastpixel_exclusions', ['type' => 'array']);
            register_setting(FASTPIXEL_TEXTDOMAIN, 'fastpixel_params_exclusions', ['type' => 'array']);
            // Register a new section in the "settings" page.
            add_settings_section(
                'fastpixel_settings_section',
                '',
                false,
                FASTPIXEL_TEXTDOMAIN
            );

            add_settings_field(
                'fastpixel_serve_stale',
                esc_html__('Serve stale', 'fastpixel-website-accelerator'),
                [$this, 'field_serve_stale_cb'],
                FASTPIXEL_TEXTDOMAIN,
                'fastpixel_settings_section'
            );
            add_settings_field(
                'fastpixel_display_cached_for_logged',
                esc_html__('Logged-in users', 'fastpixel-website-accelerator'),
                [$this, 'fastpixel_display_cached_for_logged_cb'],
                FASTPIXEL_TEXTDOMAIN,
                'fastpixel_settings_section'
            );
            add_settings_field(
                'fastpixel_exclusions',
                esc_html__('Page Exclusions', 'fastpixel-website-accelerator'),
                [$this, 'field_exclusions_cb'],
                FASTPIXEL_TEXTDOMAIN,
                'fastpixel_settings_section'
            );
            add_settings_field(
                'fastpixel_params_exclusions',
                esc_html__('Parameter Exclusions', 'fastpixel-website-accelerator'),
                [$this, 'field_params_exclusions_cb'],
                FASTPIXEL_TEXTDOMAIN,
                'fastpixel_settings_section'
            );
        }

        function field_serve_stale_cb($args)
        {
            // Get the value of the setting we've registered with register_setting()
            $serve_stale = $this->functions->get_option('fastpixel_serve_stale');
            ?>
            <input id="fastpixel_serve_stale" type="checkbox" name="fastpixel_serve_stale" value="1" <?php echo checked($serve_stale); ?>> <span class="fastpixel-field-desc"><?php esc_html_e('Serve content from the cache while regenerating the optimized page version.', 'fastpixel-website-accelerator'); ?></span>
            <?php
        }

        function fastpixel_display_cached_for_logged_cb($args)
        {
            // Get the value of the setting we've registered with register_setting()
            $enabled = $this->functions->get_option('fastpixel_display_cached_for_logged');
            ?>
            <input id="fastpixel_display_cached_for_logged" type="checkbox" name="fastpixel_display_cached_for_logged" value="1" <?php echo checked($enabled); ?>> <span class="fastpixel-field-desc"><?php esc_html_e('Display cached pages for logged-in users.', 'fastpixel-website-accelerator'); ?></span>
            <?php
        }

        function field_exclusions_cb($args)
        {
            // Get the value of the setting we've registered with register_setting()
            $exclusions = $this->functions->get_option('fastpixel_exclusions');
            ?>
            <textarea id="fastpixel_exclusions" name="fastpixel_exclusions" rows="7" cols="100" class="fastpixel-textarea"><?php echo esc_textarea($exclusions); ?></textarea>
            <span class="fastpixel-field-desc fastpixel-textarea-desc"><?php 
            /* translators: %s used to display "br" tag, nothing to translate */
            echo sprintf(esc_html__('Page URLs that should be excluded from caching. Each URL should be added on a new line.%sExample: /checkout/', 'fastpixel-website-accelerator'), '<br/>'); ?></span>
            <?php
        }

        function field_params_exclusions_cb($args)
        {
            // Get the value of the setting we've registered with register_setting()
            $params_exclusions = $this->functions->get_option('fastpixel_params_exclusions');
            ?>
            <textarea id="fastpixel_params_exclusions" name="fastpixel_params_exclusions" rows="7" cols="100" class="fastpixel-textarea"><?php echo esc_textarea($params_exclusions); ?></textarea>
            <span class="fastpixel-field-desc fastpixel-textarea-desc"><?php
            /* translators: %s used to display "br" tag, nothing to translate */
            echo sprintf(esc_html__('Request parameters that should exclude pages from caching. Each parameter should be added on a new line..%sExample: param_name=param_value, another_param_name', 'fastpixel-website-accelerator'), '<br/>'); ?></span>
            <?php
        }
        public function save_options()
        {
            if ($_SERVER['REQUEST_METHOD'] !== 'POST' || (defined('DOING_AJAX') && DOING_AJAX) || 
                check_admin_referer('fastpixel-settings', 'fastpixel-nonce') == false ||
                empty($_POST['fastpixel-action']) || sanitize_key($_POST['fastpixel-action']) != 'save_settings') {
                return;
            }
            $stale = isset($_POST['fastpixel_serve_stale']) && 1 == sanitize_text_field($_POST['fastpixel_serve_stale']) ? 1 : 0;
            $this->functions->update_option('fastpixel_serve_stale', $stale);
            $dcflin = isset($_POST['fastpixel_display_cached_for_logged']) && 1 == sanitize_text_field($_POST['fastpixel_display_cached_for_logged']) ? 1 : 0;
            $this->functions->update_option('fastpixel_display_cached_for_logged', $dcflin);
            //saving excludes and removing existing files if they exist
            $this->save_excludes();
            $this->functions->update_option('fastpixel_params_exclusions', sanitize_textarea_field($_POST['fastpixel_params_exclusions']));
            $notices = FASTPIXEL_Notices::get_instance();
            //displaying notice
            $notices->add_flash_notice(esc_html__('Settings have been saved', 'fastpixel-website-accelerator'), 'success', false);
        }

        public function save_excludes() {
            //added extra check to avoid pcp validation notice
            if (check_admin_referer('fastpixel-settings', 'fastpixel-nonce') == false) {
                return;
            }
            //getting home url for proper url generation
            $home_url = rtrim(home_url(), '/');
            //getting old value
            $old_values = $this->functions->get_option('fastpixel_exclusions');
            //getting new(submitted) value
            $new_values = sanitize_textarea_field($_POST['fastpixel_exclusions']);
            //comparing values and saving only when differs
            if ($old_values != $new_values) {
                //updating value
                $this->functions->update_option('fastpixel_exclusions', sanitize_textarea_field($_POST['fastpixel_exclusions']));
                //checking if new value is string
                if (is_string($new_values)) {
                    //making array of values from string
                    $excludes = explode("\r\n", $new_values);
                    //deleting existing cache files for excludes 
                    foreach($excludes as $exclude) {
                        $exclude_url = new FASTPIXEL_Url(null, $home_url . $exclude);
                        if (!empty($exclude)) {
                            $this->functions->delete_cached_files($exclude_url->get_url_path());
                        }
                    }
                }
            }
        }
    }
    new FASTPIXEL_Tab_Settings;
}