<?php
namespace FASTPIXEL;
defined('ABSPATH') || exit;

if (!class_exists('FASTPIXEL\FASTPIXEL_UI_Tab')) {
    abstract class FASTPIXEL_UI_Tab extends FASTPIXEL_Backend_Controller
    {
        protected $enabled = true;
        protected $order = 0;
        protected $name;
        protected $slug;

        function __construct()
        {
            parent::__construct();
            //registering tab settings
            $this->settings();
            //adding tab to UI
            $ui = FASTPIXEL_UI::get_instance();
            $ui->add_tab($this);
        }

        function enable() {
            $this->enabled = true;
        }
        
        function disable() {
            $this->enabled = false;
        }

        function is_enabled() {
            return $this->enabled;
        }

        function get_order() {
            return $this->order;
        }
        function get_name() {
            return $this->name;
        }

        function get_slug() {
            return $this->slug;
        }

        function get_link() {
            if (!is_multisite()) {
                return esc_url(admin_url('admin.php?page=' . FASTPIXEL_TEXTDOMAIN . '&tab=' . $this->slug));
            } else {
                return esc_url(network_admin_url('admin.php?page=' . FASTPIXEL_TEXTDOMAIN . '&tab=' . $this->slug));
            }
        }

        abstract function settings();

        function view() {
            if (file_exists(FASTPIXEL_PLUGIN_DIR . DIRECTORY_SEPARATOR . 'inc' . DIRECTORY_SEPARATOR . 'backend' . DIRECTORY_SEPARATOR . 'views' . DIRECTORY_SEPARATOR . $this->slug . '.php')) {
                include_once(FASTPIXEL_PLUGIN_DIR . DIRECTORY_SEPARATOR . 'inc' . DIRECTORY_SEPARATOR . 'backend' . DIRECTORY_SEPARATOR . 'views' . DIRECTORY_SEPARATOR . $this->slug . '.php');
            }
        }
    }
}

if (!class_exists('FASTPIXEL\FASTPIXEL_UI')) {
    abstract class FASTPIXEL_UI extends FASTPIXEL_Backend_Controller {

        protected static $instance;
        protected $tabs = [];
        protected $icon_url;
        protected $current_tab = 'cache_status';
        protected $allowed_tags = [
            'h2' => [
                'class' => []
            ],
            'div' => [
                'class' => []
            ],
            'a' => [
                'href' => [],
                'target' => []
            ]
        ];

        function __construct() {
            parent::__construct();
            self::$instance = $this;
            if (!\is_user_logged_in()) {
                return;
            }
            if (file_exists(FASTPIXEL_PLUGIN_DIR . DIRECTORY_SEPARATOR . 'icons' . DIRECTORY_SEPARATOR . 'FastPixel-Happy.svg')) {
                $this->icon_url = esc_url(FASTPIXEL_PLUGIN_URL . 'icons/FastPixel-Happy.svg');
            }
            global $pagenow;
            if ($pagenow == 'plugins.php') {
                add_filter('plugin_action_links', [$this, 'plugin_action_links'], 10, 2);
            }
            wp_enqueue_style('fastpixel_admin_menu_css', FASTPIXEL_PLUGIN_URL . 'inc/backend/assets/admin-menu.css?' . time(), [], FASTPIXEL_VERSION);
            //initializing tabs only when page is opened
            // phpcs:ignore WordPress.Security.NonceVerification.Recommended -- wordpress page is accessed without any nonces, no data is posted.
            $page = isset($_GET['page']) ? sanitize_key($_GET['page']) : false;
            if ($pagenow == 'admin.php' && $page && in_array($page, [FASTPIXEL_TEXTDOMAIN, FASTPIXEL_TEXTDOMAIN . '-settings'])) {
                wp_enqueue_style('fastpixel_admin_css', FASTPIXEL_PLUGIN_URL . 'inc/backend/assets/backend.css?' . time(), [], FASTPIXEL_VERSION);
                add_action('admin_init', [$this, 'init_tabs']);
                add_action('load-toplevel_page_' . FASTPIXEL_TEXTDOMAIN, [$this, 'screen_options']);
                add_filter('set-screen-option', [$this, 'save_screen_options'], 10, 3);
            }
            //adding scripts only when page is opened
            add_action('admin_enqueue_scripts', function () {
                wp_register_script('fastpixel-backend', FASTPIXEL_PLUGIN_URL . 'inc/backend/assets/backend.js?' . time(), array('jquery-ui-core', 'jquery-ui-tabs'), FASTPIXEL_VERSION, false);
                wp_localize_script('fastpixel-backend', 'fastpixel_backend', [
                    'ajax_url'                 => admin_url('admin-ajax.php'), 
                    'nonce'                    => wp_create_nonce('cache_status_nonce'),
                    'deactivate_plugin_nonce'  => wp_create_nonce("fastpixel_deactivate_plugin"),
                    'delete_cached_files_text' => esc_html__('Delete Cached Files', 'fastpixel-website-accelerator'),
                    'delete_cached_files_link' => sprintf('admin-post.php?action=%1$s&nonce=%2$s&post_id=', 'fastpixel_admin_delete_cached', wp_create_nonce('cache_status_nonce')),
                    'cache_now_text'           => esc_html__('Cache Now', 'fastpixel-website-accelerator'),
                    'purge_cache_text'         => esc_html__('Purge Cache', 'fastpixel-website-accelerator'),
                    'purge_post_link'          => sprintf('admin-post.php?action=%1$s&nonce=%2$s&post_id=', 'fastpixel_admin_purge_post_cache', wp_create_nonce('cache_status_nonce')),
                    'deactivate_plugin_text'   => esc_html__('DEACTIVATED', 'fastpixel-website-accelerator')
                ]);
                global $pagenow;
                // phpcs:ignore WordPress.Security.NonceVerification.Recommended -- wordpress page is accessed without any nonces, no data is posted.
                $page = isset($_GET['page']) ? sanitize_key($_GET['page']) : false;
                if ($pagenow == 'admin.php' && $page && in_array($page, [FASTPIXEL_TEXTDOMAIN, FASTPIXEL_TEXTDOMAIN.'-settings'])) {
                    wp_enqueue_script('fastpixel-backend');
                }
            });
            add_action('admin_init', [$this, 'deactivation_popup']);
        }

        public static function get_instance()
        {
            return self::$instance;
        }

        public function init_tabs()
        {
            if ($handle = opendir(FASTPIXEL_PLUGIN_DIR . DIRECTORY_SEPARATOR . 'inc' . DIRECTORY_SEPARATOR . 'backend' . DIRECTORY_SEPARATOR . 'controllers' . DIRECTORY_SEPARATOR . 'tabs')) {
                while (false !== ($entry = readdir($handle))) {
                    if (!in_array($entry, ['.', '..'])) {
                        try {
                            include_once(FASTPIXEL_PLUGIN_DIR . DIRECTORY_SEPARATOR . 'inc' . DIRECTORY_SEPARATOR . 'backend' . DIRECTORY_SEPARATOR . 'controllers' . DIRECTORY_SEPARATOR . 'tabs' . DIRECTORY_SEPARATOR . $entry);
                        } catch (\Exception $e) {
                            FASTPIXEL_DEBUG::log('Exception message -> ', $e->getMessage());
                        }
                    }
                }
                closedir($handle);
            }
            usort(
                $this->tabs,
                function ($a, $b) {
                    return $a->get_order() > $b->get_order() ? 1 : ($a->get_order() == $b->get_order() ? 0 : -1);
                }
            );
        }

        public function page_title($page_name) {
            switch ($page_name) {
            case 'status':
                return '<h2 class="fastpixel-page-title">' . esc_html__('FastPixel', 'fastpixel-website-accelerator') . '</h2>';
                break;
            case 'settings':
                return '<h2 class="fastpixel-page-title">' . esc_html__('FastPixel Settings', 'fastpixel-website-accelerator') . '</h2>';
                break;
            default:
                return '<h2 class="fastpixel-page-title">' . esc_html__('FastPixel', 'fastpixel-website-accelerator') . '</h2>';
                break;
            }
        }

        public function add_tab($tab)
        {
            $this->tabs[] = $tab;
        }

        public function get_tabs()
        {
            return $this->tabs;
        }

        function plugin_action_links($plugin_actions, $plugin_file)
        {
            if (!$this->check_capabilities()) {
                return;
            }
            if (defined('FASTPIXEL_PLUGIN_FILE') && strpos(FASTPIXEL_PLUGIN_FILE, $plugin_file)) {
                return array_merge(
                    array(
                        'settings' => '<a href="' . esc_url(\admin_url('admin.php?page=' . FASTPIXEL_TEXTDOMAIN . '-settings')) . '">' . esc_html__('Settings', 'fastpixel-website-accelerator') . '</a>',
                    ),
                    $plugin_actions
                );
            }
            return $plugin_actions;
        }

        abstract function admin_bar();
        abstract function admin_menu();

        public function get_icon() {
            return '<div class="fastpixel-top-menu-icon"><img src="' . $this->icon_url . '" /></div>';
        }

        public function submenu() {
            return '<div class="fastpixel-submenu">
                    <a href="https://fastpixel.io/pricing/" target="_blank">' .esc_html__('Upgrade Plan', 'fastpixel-website-accelerator') . '</a>
                    <a href="https://fastpixel.io/docs/" target="_blank">' .esc_html__('Knowledge Base', 'fastpixel-website-accelerator') . '</a>
                    <a href="https://fastpixel.io/#contact" target="_blank">' .esc_html__('Contact Support', 'fastpixel-website-accelerator') . '</a>
                    <a href="https://dash.fastpixel.io/" target="_blank">' .esc_html__('FastPixel Account', 'fastpixel-website-accelerator') . '</a>
                    </div>';
        }

        // add screen options
        function screen_options()
        {
            $screen = get_current_screen();
            // get out of here if we are not on our settings page
            if (!is_object($screen) || $screen->id != 'toplevel_page_' . FASTPIXEL_TEXTDOMAIN) {
                return;
            }
            $args = array(
                'label'   => esc_html__('Number of items per page', 'fastpixel-website-accelerator'),
                'default' => 20,
                'option'  => 'fastpixel_per_page'
            );
            add_screen_option('per_page', $args);
        }

        function save_screen_options($status, $option, $value)
        {
            if ($option == 'fastpixel_per_page') {
                // Process and save the value as needed
                update_user_meta(get_current_user_id(), 'fastpixel_per_page', $value);
            }
            return $status;
        }

        public function status_page()
        {
            if (!$this->check_capabilities()) {
                return;
            }
            echo '<div class="wrap fastpixel-website-accelerator-wrap">';
            echo wp_kses($this->page_title('status'), $this->allowed_tags);
            echo wp_kses($this->submenu(), $this->allowed_tags);
            foreach ($this->tabs as $tab) {
                if (!in_array($tab->get_slug(), array('cache_status')) || !$tab->is_enabled()) {
                    continue;
                }
                $tab->view();
            }
            echo '</div>';
        }

        public function settings_page()
        {
            if (!$this->check_capabilities()) {
                return;
            }
            $page_tabs = array('settings', 'javascript', 'images', 'fonts', 'diagnostics');
            echo '<div class="wrap fastpixel-website-accelerator-wrap">';
            echo wp_kses($this->page_title('settings'), $this->allowed_tags);
            echo wp_kses($this->submenu(), $this->allowed_tags);
            echo '<div id="fastpixel-tabs"><ul>';
            foreach ($this->tabs as $tab) {
                if (!in_array($tab->get_slug(), $page_tabs) || !$tab->is_enabled()) {
                    continue;
                }
                echo '<li data-slug="' . esc_attr($tab->get_slug()) . '"><a class="fastpixel-tab" href="#' . esc_attr($tab->get_slug()) . '">' . wp_kses_post($tab->get_name()) . '</a></li>';
            }
            echo '</ul><form method="post">';
            wp_nonce_field('fastpixel-settings', 'fastpixel-nonce', false);
            echo '<input type="hidden" name="fastpixel-action" value="save_settings" />';
            foreach ($this->tabs as $tab) {
                if (!in_array($tab->get_slug(), $page_tabs) || !$tab->is_enabled()) {
                    continue;
                }
                echo '<div id="' . esc_attr($tab->get_slug()) . '" class="fastpixel-tab-content">';
                $tab->view();
                echo '</div>';
            }
            echo '</form></div>';
        }
        public function deactivation_popup()
        {
            global $pagenow;
            if (strcmp($pagenow, 'plugins.php') == 0) {
                add_action('admin_enqueue_scripts', function () {
                    wp_enqueue_style('fastpixel-popup', FASTPIXEL_PLUGIN_URL . 'inc/backend/assets/backend.css', [], FASTPIXEL_VERSION);
                    wp_enqueue_script('fastpixel-popup', FASTPIXEL_PLUGIN_URL . 'inc/backend/assets/popup.js', ['jquery'], FASTPIXEL_VERSION);
                    wp_localize_script('fastpixel-popup', 'fastpixel_popup', [
                        'btn_deactivate'       => __('Continue Deactivation', 'fastpixel-webisite-accelerator'),
                        'btn_deactivate_link'  => wp_nonce_url(admin_url('plugins.php?action=deactivate&plugin='.plugin_basename(FASTPIXEL_PLUGIN_FILE)), 'deactivate-plugin_' . plugin_basename(FASTPIXEL_PLUGIN_FILE)),
                        'btn_delete'           => __('Delete files and Deactivate', 'fastpixel-webisite-accelerator'),
                        'btn_delete_link'      => wp_nonce_url(admin_url('plugins.php?action=deactivate&plugin='.plugin_basename(FASTPIXEL_PLUGIN_FILE)) . '&fastpixel-action=delete_cached_files', 'deactivate-plugin_' . plugin_basename(FASTPIXEL_PLUGIN_FILE)),
                        'deactivation_message' => __('Do you want to delete cached files before plugin deactivation?', 'fastpixel-webisite-accelerator'),
                    ]);
                });
            }
        }
    }
}

if (!class_exists('FASTPIXEL\FASTPIXEL_UI_Single')) {
    class FASTPIXEL_UI_Single extends FASTPIXEL_UI
    {

        function __construct()
        {
        parent::__construct();
            add_action('admin_bar_menu', [$this, 'admin_bar'], 90);
            add_action('admin_menu', [$this, 'admin_menu']);
        }

        function admin_menu()
        {
            if (!$this->check_capabilities()) {
                return;
            }
            add_menu_page(FASTPIXEL_NAME, FASTPIXEL_NAME, 'manage_options', FASTPIXEL_TEXTDOMAIN, [$this, 'status_page'], ($this->icon_url ?: ''), 99);
            add_submenu_page(FASTPIXEL_TEXTDOMAIN, esc_html__('Cache Status', 'fastpixel-website-accelerator'), esc_html__('Cache Status', 'fastpixel-website-accelerator'), 'manage_options', FASTPIXEL_TEXTDOMAIN, [$this, 'status_page'], 1);
            add_submenu_page(FASTPIXEL_TEXTDOMAIN, esc_html__('Settings', 'fastpixel-website-accelerator'), esc_html__('Settings', 'fastpixel-website-accelerator'), 'manage_options', FASTPIXEL_TEXTDOMAIN . '-settings', [$this, 'settings_page'], 2);
            //we can display purge all button in left menu if client needs it and define it in wp-config
            if (defined('FASTPIXEL_LEFT_MENU_PURGE') && FASTPIXEL_LEFT_MENU_PURGE) {
                add_submenu_page(FASTPIXEL_TEXTDOMAIN, esc_html__('Purge All Cache', 'fastpixel-website-accelerator'), esc_html__('Purge All Cache', 'fastpixel-website-accelerator'), 'manage_options', esc_url(wp_nonce_url(admin_url('admin.php?page=' . FASTPIXEL_TEXTDOMAIN . '&fastpixel-action=fastpixel_purge_cache'), 'fastpixel_purge_cache', 'fastpixel_cache_nonce')), null, 3);
            }
        }

        function admin_bar()
        {
            if (!$this->check_capabilities()) {
                return;
            }
            global $wp_admin_bar;
            $wp_admin_bar->add_menu([
                'id'     => 'fastpixel-top-' . FASTPIXEL_TEXTDOMAIN . '-menu',
                'href'   => esc_url(admin_url('admin.php?page=' . FASTPIXEL_TEXTDOMAIN)),
                'title'  => ($this->get_icon() ? : '') .esc_html__('FastPixel Cache', 'fastpixel-website-accelerator'),
                'meta'   => ['title' => esc_html__('FastPixel Cache', 'fastpixel-website-accelerator')],
            ]);
            $wp_admin_bar->add_node([
                'id'     => 'fastpixel-top-' . FASTPIXEL_TEXTDOMAIN . '-status',
                'parent' => 'fastpixel-top-' . FASTPIXEL_TEXTDOMAIN . '-menu',
                'href'   => esc_url(admin_url('admin.php?page=' . FASTPIXEL_TEXTDOMAIN)),
                'title'  => esc_html__('Cache Status', 'fastpixel-website-accelerator'),
            ]);
            $wp_admin_bar->add_node([
                'id'     => 'fastpixel-top-' . FASTPIXEL_TEXTDOMAIN . '-settings',
                'parent' => 'fastpixel-top-' . FASTPIXEL_TEXTDOMAIN . '-menu',
                'href'   => esc_url(admin_url('admin.php?page=' . FASTPIXEL_TEXTDOMAIN .'-settings')),
                'title'  => esc_html__('Settings', 'fastpixel-website-accelerator'),
            ]);
            $wp_admin_bar->add_node([
                'id'     => 'fastpixel-top-' . FASTPIXEL_TEXTDOMAIN . '-purge-cache',
                'parent' => 'fastpixel-top-' . FASTPIXEL_TEXTDOMAIN . '-menu',
                'href'   => esc_url(wp_nonce_url(admin_url('admin.php?page=' . FASTPIXEL_TEXTDOMAIN . '&fastpixel-action=fastpixel_purge_cache'), 'fastpixel_purge_cache', 'fastpixel_cache_nonce')),
                'title'  => esc_html__('Purge All Cache', 'fastpixel-website-accelerator'),
            ]);
        }

    }
}

if (!class_exists('FASTPIXEL\FASTPIXEL_UI_Multi')) {
    class FASTPIXEL_UI_Multi extends FASTPIXEL_UI
    {
        function __construct()
        {
            parent::__construct();
            self::$instance = $this;
            add_action('admin_bar_menu', [$this, 'admin_bar'], 90);
            add_action('admin_menu', [$this, 'admin_menu']);
            add_action('network_admin_menu', [$this, 'network_admin_menu']);

        }

        public static function get_instance()
        {
            if (empty(self::$instance)) {
                self::$instance = new FASTPIXEL_UI_Multi();
            }
            return self::$instance;
        }

        function admin_menu() {
            add_menu_page(FASTPIXEL_NAME, FASTPIXEL_NAME, 'manage_options', FASTPIXEL_TEXTDOMAIN, [$this, 'status_page'], ($this->icon_url ?: ''), 99);
            add_submenu_page(FASTPIXEL_TEXTDOMAIN, esc_html__('Cache Status', 'fastpixel-website-accelerator'), esc_html__('Cache Status', 'fastpixel-website-accelerator'), 'manage_options', FASTPIXEL_TEXTDOMAIN, [$this, 'status_page'], 1);
            //we can display purge all button in left menu if client needs it and define it in wp-config
            if (defined('FASTPIXEL_LEFT_MENU_PURGE') && FASTPIXEL_LEFT_MENU_PURGE) {
                add_submenu_page(FASTPIXEL_TEXTDOMAIN, esc_html__('Purge All Cache', 'fastpixel-website-accelerator'), esc_html__('Purge All Cache', 'fastpixel-website-accelerator'), 'manage_options', esc_url(wp_nonce_url(admin_url('admin.php?page=' . FASTPIXEL_TEXTDOMAIN . '&fastpixel-action=fastpixel_purge_cache'), 'fastpixel_purge_cache', 'fastpixel_cache_nonce')), null, 2);
            }
        }

        function network_admin_menu()
        {
            add_menu_page(FASTPIXEL_NAME, FASTPIXEL_NAME, 'manage_network', FASTPIXEL_TEXTDOMAIN . '-settings', [$this, 'settings_page'], ($this->icon_url ? : null), 99);
        }

        function admin_bar()
        {
            if (!$this->check_capabilities()) {
                return;
            }
            global $wp_admin_bar;
            $wp_admin_bar->add_node([
                'id'     => 'fastpixel-top-' . FASTPIXEL_TEXTDOMAIN . '-menu',
                'href'   => esc_url(admin_url('admin.php?page=' . FASTPIXEL_TEXTDOMAIN)),
                'title'  => ($this->get_icon() ? : '') . esc_html__('FastPixel Cache', 'fastpixel-website-accelerator'),
                'meta'   => ['title' => esc_html__('FastPixel Cache', 'fastpixel-website-accelerator')]
            ]);
            //we can add status link when not in network admin
            if (!is_network_admin()) {
                $wp_admin_bar->add_node([
                    'id'     => 'fastpixel-top-' . FASTPIXEL_TEXTDOMAIN . '-status',
                    'parent' => 'fastpixel-top-' . FASTPIXEL_TEXTDOMAIN . '-menu',
                    'href'   => esc_url(admin_url('admin.php?page=' . FASTPIXEL_TEXTDOMAIN)),
                    'title'  => esc_html__('Cache Status', 'fastpixel-website-accelerator'),
                ]);
            }
            $wp_admin_bar->add_node([
                'id'     => 'fastpixel-top-' . FASTPIXEL_TEXTDOMAIN . '-settings',
                'parent' => 'fastpixel-top-' . FASTPIXEL_TEXTDOMAIN . '-menu',
                'href'   => esc_url(network_admin_url('admin.php?page=' . FASTPIXEL_TEXTDOMAIN . '-settings')),
                'title'  => esc_html__('Settings', 'fastpixel-website-accelerator'),
            ]);
            $wp_admin_bar->add_node([
                'id'     => 'fastpixel-top-' . FASTPIXEL_TEXTDOMAIN . '-purge-cache',
                'parent' => 'fastpixel-top-' . FASTPIXEL_TEXTDOMAIN . '-menu',
                'href'   => esc_url(wp_nonce_url(admin_url('admin.php?page=' . FASTPIXEL_TEXTDOMAIN . '&amp;fastpixel-action=fastpixel_purge_cache'), 'fastpixel_purge_cache', 'fastpixel_cache_nonce')),
                'title'  => esc_html__('Purge All Cache', 'fastpixel-website-accelerator'),
            ]);
        }

        public function check_capabilities()
        {
            // check user capabilities
            if (!current_user_can('manage_network')) {
                return false;
            }
            return true;
        }
    }
}

add_action('init', function () {
    if (is_multisite()) {
        new FASTPIXEL_UI_Multi();
    } else {
        new FASTPIXEL_UI_Single();
    }
});