<?php
namespace FASTPIXEL;
use WP_Query;
defined('ABSPATH') || exit;

if (!class_exists('WP_List_Table')) {
    require_once(ABSPATH . 'wp-admin/includes/class-wp-list-table.php');
}

if (!class_exists('FASTPIXEL\FASTPIXEL_Posts_Table')) {
    class FASTPIXEL_Posts_Table extends \WP_List_Table
    {
        protected $post_types = [];
        protected $selected_post_type = 'page';
        protected $posts_order = 'ID';
        protected $posts_orderby = 'asc';
        protected $functions;
        protected $serve_stale;
        protected $nonce;
        protected $be_functions;
        protected $search;

        public function __construct() {
            parent::__construct();
            $this->functions    = FASTPIXEL_Functions::get_instance();
            $this->be_functions = FASTPIXEL_Backend_Functions::get_instance();
            $this->serve_stale  = $this->functions->get_option('fastpixel_serve_stale');
            $this->post_types   = get_post_types([], 'objects');
            // phpcs:ignore WordPress.Security.NonceVerification.Recommended -- wordpress page is accessed without any nonces.
            if (isset($_GET['ptype'])) { 
                $this->selected_post_type = sanitize_key($_GET['ptype']); //phpcs:ignore
            }
            // phpcs:ignore WordPress.Security.NonceVerification.Recommended -- wordpress page is accessed without any nonces.
            if (isset($_REQUEST['order']) ) {
                $order = sanitize_key($_REQUEST['order']); //phpcs:ignore
                if (is_string($order) && in_array(strtolower($order), array('asc', 'desc'))) {
                    $this->posts_order = $order;
                }
            }
            // phpcs:ignore WordPress.Security.NonceVerification.Recommended -- wordpress page is accessed without any nonces.
            if (isset($_REQUEST['orderby'])) {
                $orderby = sanitize_key($_REQUEST['orderby']); //phpcs:ignore
                if (is_string($orderby) && in_array(strtolower($orderby), array('id'))) {
                    $this->posts_orderby = $orderby;
                }
            }
            // phpcs:ignore WordPress.Security.NonceVerification.Recommended -- wordpress page is accessed without any nonces.
            if (isset($_REQUEST['s'])) {
                $search = sanitize_text_field($_REQUEST['s']); //phpcs:ignore
                if (is_string($search)) {
                    $this->search = $search;
                }
            }
            $this->nonce = wp_create_nonce('cache_status_nonce');
            $this->process_bulk_action();
        }

        public function get_columns()
        {
            return [
                'cb'           => '<input type="checkbox" />',
                'ID'           => esc_html__('ID', 'fastpixel-website-accelerator'),
                'post_title'   => esc_html__('Title', 'fastpixel-website-accelerator'),
                'url'          => esc_html__('URL', 'fastpixel-website-accelerator'),
                'post_status'  => esc_html__('Post Status', 'fastpixel-website-accelerator'),
                'cache_status' => esc_html__('Cache Status', 'fastpixel-website-accelerator')
            ];
        }


        // Adding action links to column
        function column_url($item)
        {
            //setting links
            $link = get_permalink($item['ID']);
            $purge_id = $item['ID'];
            //setting links for homepage
            if ($item['ID'] == null && $item['post_title'] == 'Homepage') {
                $purge_id = 'homepage';
                $link = get_home_url();
            }
            $edit_link = sprintf('post.php?post=%1$d&action=%2$s', $item['ID'], 'edit');
            $purge_link = sprintf('admin-post.php?action=%1$s&nonce=%2$s&post_id=%3$s', $purge_id == 'homepage' ? 'fastpixel_admin_purge_homepage_cache' : 'fastpixel_admin_purge_post_cache', $this->nonce, $purge_id);
            //actions
            if ($item['cachestatus'] != 'cached') {
                $purge_cache_link = sprintf('<a class="fastpixel-purge-single-post" data-post-id="%1$s" href="%2$s">' . esc_html__('Cache Now', 'fastpixel-website-accelerator') . '</a>', $purge_id, esc_url($purge_link));
            } else {
                $purge_cache_link = sprintf('<a class="fastpixel-purge-single-post" data-post-id="%1$s" href="%2$s">' . esc_html__('Purge Cache', 'fastpixel-website-accelerator') . '</a>', $purge_id, esc_url($purge_link));
            }
            $actions = array(
                'view' => sprintf('<a href="%s" target="_blank">' .esc_html__('Preview', 'fastpixel-website-accelerator') . '</a>', esc_url($link)),
                'edit' => sprintf('<a href="%s">' .esc_html__('Edit', 'fastpixel-website-accelerator') . '</a>', esc_url($edit_link)),
                'purge_cache' => $purge_cache_link
            );
            //adding delete cached files action for stale items
            if ($item['cachestatus'] == 'stale' || 
                (in_array($item['cachestatus'], array('excluded', 'error')) && 
                (isset($item['html_created_time']) && $item['html_created_time'] > 0))) {
                $delete_link = sprintf('admin-post.php?action=%1$s&nonce=%2$s&post_id=%3$d', 'fastpixel_admin_delete_cached', $this->nonce, $purge_id);
                $actions['delete_cached'] = sprintf('<a class="fastpixel-delete-cached-files-single-post" data-post-id="%1$s" href="%s">' .esc_html__('Delete Cached Files', 'fastpixel-website-accelerator') . '</a>', $purge_id, esc_url($delete_link));
            }
            //removing edit button for homepage, when post_id is not present
            if ($purge_id == 'homepage') {
                unset($actions['edit']);
            }
            if ($item['cachestatus'] == 'excluded') {
                unset($actions['purge_cache']);
            }
            return sprintf('%1$s %2$s', esc_html($item['url']), $this->row_actions($actions));
        }

        public function prepare_items()
        {
            $columns = $this->get_columns();
            $hidden = array();
            $sortable = $this->get_sortable_columns();
            $this->_column_headers = array($columns, $hidden, $sortable);

            /* pagination */
            $per_page = $this->get_items_per_page('fastpixel_per_page', 20);
            $current_page = $this->get_pagenum();
            $offset = ($current_page - 1) * $per_page;
            $args = array(
                'posts_per_page' => $per_page,
                'offset'         => $offset,
                'post_type'      => $this->selected_post_type,
                'post_status'    => $this->selected_post_type == 'attachment' ? 'any' : 'publish'
            );
            if ($this->posts_order && in_array(strtolower($this->posts_order), array('asc', 'desc'))) {
                $args['order'] = $this->posts_order;
            }
            if ($this->posts_orderby && in_array(strtolower($this->posts_orderby), array('id'))) {
                $args['orderby'] = $this->posts_orderby;
            }
            if (!empty($this->search)) {
                $args['s'] = $this->search;
            }
            $extra_pages = [];
            if ($this->selected_post_type == 'page') {
                $page_on_front  = get_option('page_on_front');
                //if static page is not set, adding homepage to list
                if ($page_on_front == 0 && $current_page == 1) {
                    $cache_status = $this->be_functions->cache_status_display(get_home_url());
                    $extra_pages[] = [
                        'ID'           => 'homepage',
                        'post_title'   => esc_html__('Homepage', 'fastpixel-website-accelerator'),
                        'url'          => get_home_url(),
                        'cache_status' => $cache_status['status_display'],
                        'cachestatus'  => $cache_status['status'],
                        'post_status'  => '<b>published</b>'
                    ];
                }
            }
            $this->items = array_merge($extra_pages, $this->get_table_data($args));
        }

        protected function get_table_data($args)
        {

            // If no orderby, default ID
            if (empty($args['orderby'])) {
                $args['orderby'] = 'ID';
            }
            // If no order, default to asc
            if (empty($args['order'])) {
                $args['order'] = 'asc';
            }
            if (!isset($args['post_type']) || empty($args['post_type'])) {
                $args['post_type'] = 'post';
            }

            $wp_query = new WP_Query($args);
            $results = $wp_query->get_posts();

            $return_posts = [];
            foreach ($results as $result) {
                $url = get_permalink($result->ID);
                $cache_status = $this->be_functions->cache_status_display($url);
                $return_posts[] = [
                    'ID'                => $result->ID,
                    'post_title'        => $result->post_title,
                    'url'               => urldecode($url),
                    'cache_status'      => $cache_status['status_display'],
                    'cachestatus'       => $cache_status['status'],
                    'post_status'       => $result->post_status == 'publish' ? '<b>' . __('published', 'fastpixel-website-accelerator') . '</b>' : $result->post_status,
                    'html_created_time' => $cache_status['html_created_time'],

                ];
            }
            $this->set_pagination_args(
                array(
                    'total_items' => $wp_query->found_posts,
                    'per_page'    => $args['posts_per_page'],
                    'total_pages' => ceil($wp_query->found_posts / $args['posts_per_page']) // use ceil to round up
                )
            );
            return $return_posts;
        }

        function column_default($item, $column_name)
        {
            switch ($column_name) {
                case 'title':
                case 'url':
                case 'status':
                default:
                    return $item[$column_name];
            }
        }

        protected function get_sortable_columns()
        {
            $sortable_columns = array(
                'ID'   => array('ID', false)
            );
            return $sortable_columns;
        }

        protected function display_tablenav($which)
        {
            if ($which == 'top') {
                // phpcs:ignore WordPress.Security.NonceVerification.Recommended -- wordpress page is accessed without any nonces, no data is posted.
                $order = isset($_GET['order']) ? sanitize_key($_GET['order']) : false; //phpcs:ignore
                // phpcs:ignore WordPress.Security.NonceVerification.Recommended -- wordpress page is accessed without any nonces, no data is posted.
                $orderby = isset($_GET['orderby']) ? sanitize_key($_GET['orderby']) : false; //phpcs:ignore
                echo '<form method="get" action="' . esc_url(admin_url('admin.php?page=' . FASTPIXEL_TEXTDOMAIN)) . '">'; } ?>
                <?php $this->display_search($which); ?>
                <div class="tablenav <?php echo esc_attr($which); ?>">
                    <input type="hidden" name="page" value="<?php echo esc_html(FASTPIXEL_TEXTDOMAIN); ?>" />
                    <?php echo (isset($orderby) && $orderby ? '<input type="hidden" name="orderby" value="' . esc_html($orderby) . '" />' : '')
                    . (isset($order) && $order ? '<input type="hidden" name="order" value="' . esc_html($order) . '" />' : ''); 

                    if ($this->has_items()): ?>
                        <div class="alignleft actions bulkactions">
                        <?php $this->bulk_actions($which); ?>
                        </div>
                    <?php
                    endif;
                    $this->extra_tablenav($which);
                    $this->pagination($which);
                    ?>
                    <br class="clear" />
                </div>
            <?php
            if ($which == 'bottom') { 
                echo '</form>'; 
            } 
        }

        protected function extra_tablenav($which)
        {
            switch ($which) {
                case 'top':
                    $this->display_posts_filter('posts_filter');
                    break;
                case 'bottom':
                    break;
                default:     
                    break;
            }
        }

        protected function display_posts_filter($id = false) {
            if ($id == false) {
                return;
            }
            $options = '';
            foreach ($this->post_types as $ptype) {
                if ($ptype->name == 'attachment') {
                    continue;
                }
                if ($ptype->public == true) {
                    $options .= '<option value="' . esc_attr($ptype->name) . '" '.($this->selected_post_type == $ptype->name ? 'selected="selected"' : '').'>' . esc_html($ptype->label) . '</option>';
                }
            }
            echo wp_kses('<div class="alignleft actions">
                <select name="ptype">'.$options.'</select>
                <input type="submit" value="' .esc_html__('Filter', 'fastpixel-website-accelerator') . '" class="button action"/>
                </div>', ['div' => [], 'select' => ['name' =>[]], 'option' => ['value' => [], 'selected' => []], 'input' => ['type' => [], 'value' => [], 'class' => []]]);
        }

        public function column_cb($item)
        {
            return sprintf('<input type="checkbox" name="rid[]" value="%1$s" data-status="%2$s"/>', esc_html($item['ID']), esc_html($item['cachestatus']));
        }

        public function get_bulk_actions()
        {
            $action = array(
                'reset' => esc_html__('Reset Cache', 'fastpixel-website-accelerator')
            );
            return $action;
        }

        public function process_bulk_action()
        {

            // If the delete bulk action is triggered
            $action = $this->current_action();
            $rids = [];
            // phpcs:ignore WordPress.Security.NonceVerification.Recommended -- wordpress page is accessed without any nonces, no data is posted.
            if (isset($_GET['rid']) && is_array($_GET['rid']) && count($_GET['rid']) > 0) {
                foreach ($_GET['rid'] as $rid) { //phpcs:ignore
                    $rids[] = sanitize_key($rid);
                }
            }
            if ('reset' === $action && !empty($rids)) {
                $notices = FASTPIXEL_Notices::get_instance();
                $k = 20;
                if (count($rids) <= $k) {
                    $k = count($rids);
                } else {
                    $notices->add_flash_notice(esc_html__('Max 20 posts can be sent per one request', 'fastpixel-website-accelerator'), 'notice', false);
                }
                // loop over the array of IDs and request page cache
                $be_cache = FASTPIXEL_Backend_Cache::get_instance();
                $r_count = 0;
                for ($i = 0; $i < $k; $i++) {
                    if (isset($rids[$i]) && is_numeric($rids[$i])) {
                        if ($be_cache->purge_post((int)$rids[$i]) ) {
                            $r_count ++;
                        }
                    }
                }
                $args = array(
                    'page'    => FASTPIXEL_TEXTDOMAIN, 
                    'paged'   => $this->get_pagenum(), 
                    'ptype'   => $this->selected_post_type,
                    'order'   => $this->posts_order,
                    'orderby' => $this->posts_orderby,
                    's'       => $this->search
                );
                $post_type_name = '';
                foreach ($this->post_types as $post_type) {
                    if ($post_type->name == $this->selected_post_type) {
                        if (is_object($post_type->labels)) {
                            if ($r_count == 1) {
                                $post_type_name = strtolower($post_type->labels->singular_name);
                            } else {
                                $post_type_name = strtolower($post_type->labels->name);
                            }
                        }
                        break;
                    }
                }
                
                /* translators: %1 should be a posts count, %2 post type name */
                $notices->add_flash_notice(sprintf(esc_html__('Cache has been purged for %1$d %2$s', 'fastpixel-website-accelerator'), esc_html($r_count), esc_html($post_type_name)), 'success', false);
                wp_redirect(esc_url_raw(add_query_arg($args, admin_url('admin.php'))));
                exit;
            }
        }

        function display_search($which = 'top') {
            $post_type_name = $this->post_types[$this->selected_post_type]->labels->name ? $this->post_types[$this->selected_post_type]->labels->name : __('Posts', 'fastpixel-website-accelerator');
            switch ($which) {
                case 'top': ?>
                    <p class="search-box">
	                <label class="screen-reader-text" for="post-search-input"><?php 
                    /* translators: %s is used to display Post type */
                    echo sprintf(esc_html__('Search %s', 'fastpixel-website-accelerator'), esc_html($post_type_name)); ?>:</label>
                    <input type="search" id="post-search-input" name="s" value="">
		            <input type="submit" id="search-submit" class="button" value="<?php
                    /* translators: %s is used to display Post type */
                    echo sprintf(esc_html__('Search %s', 'fastpixel-website-accelerator'), esc_html($post_type_name)); ?>"></p>
                <?php break;
                case 'bottom':
                    break;
                default:
                    break;
            }
        }
    }
}