<?php
namespace FASTPIXEL;
defined('ABSPATH') || exit;

if (!class_exists('FASTPIXEL\FASTPIXEL_Url')) {
    class FASTPIXEL_Url {
        
        private $params_to_trim = [
            "fastpixeldebug",
            "fastpixeldisable",
            "nocache",
            "epc_nocache"
        ];
        //some known tracking params list
        private $tracking_params = [
            // Google Analytics (utm_)
            "utm_source",
            "utm_medium",
            "utm_campaign",
            "utm_term",
            "utm_content",

            // HubSpot (hsa_)
            "hsa_acc",
            "hsa_ad",
            "hsa_cam",
            "hsa_grp",
            "hsa_kw",
            "hsa_mt",
            "hsa_net",
            "hsa_src",
            "hsa_tgt",

            // Facebook (fb_)
            "fbclid",
            "fb_source",
            "fb_ref",
            "fb_action_ids",
            "fb_action_types",
            "fb_campaign_id",

            // Twitter (tw_)
            "tw_campaign_name",
            "tw_campaign_id",
            "tw_creator",
            "tw_keyword",
            "tw_matchtype",
            "tw_network",
            "tw_placement",
            "tw_targeting_criteria",

            // LinkedIn (li_)
            "li_campaign",
            "li_source",
            "li_medium",
            "li_content",
            "li_term",

            // Bing (msclkid)
            "msclkid",

            // Pinterest (referrer)
            "referrer",

            // Snapchat (sc_)
            "sc_aadid",
            "sc_adset",
            "sc_campaign",
            "sc_cid",
            "sc_content",
            "sc_country",
            "sc_creative",
            "sc_l",
            "sc_medium",
            "sc_publisher",
            "sc_segment",
            "sc_source",
            "sc_term",
            "sc_video_id",

            // TikTok (tt_)
            "tt_campaign",
            "tt_content_id",
            "tt_medium",
            "tt_source",
            "tt_term",
            "tt_video_id",

            // Amazon (amzn_)
            "amzn_bkgd",
            "amzn_bkmk",
            "amzn_brand",
            "amzn_comp",
            "amzn_osd",
            "amzn_pdr",
            "amzn_ref",
            "amzn_slot",
            "amzn_wdgt",

            // Matomo (pk_)
            "pk_campaign",
            "pk_kwd",
            "pk_keyword",
            "pk_medium",
            "pk_source",
            "pk_content",
            "pk_cid",
            "pk_cpn",
            "pk_adid",

            // Mailchimp (mc_)
            "mc_cid",
            "mc_eid",

            // Crazy Egg (ce_)
            "ce_creative",
            "ce_variant",
            "ce_experiment_id",
            "ce_experiment_variation_id",

            // Kissmetrics (k_)
            "k_vid",
            "k_key",
            "kme",
            "kmc",
            "kma",
            "kmw",

            // Adobe Analytics (cid)
            "cid"
        ];
        private $original_url;
        private $url;
        private $scheme;
        private $port;
        private $host;
        private $path;
        private $query;
        private $url_path;

        public function __construct($post_id = null, $url = null) {
            if (function_exists('get_permalink') && !empty($post_id)) {
                $this->original_url = get_permalink($post_id);
                FASTPIXEL_DEBUG::log('Getting URL by $post_id', $this->original_url);
            } else if (!empty($url)) {
                $this->original_url = $url;
            } else {
                $this->original_url = $this->get_data_from_request();
                FASTPIXEL_DEBUG::log('Getting URL from Request', $this->original_url);
            }
            /*
             * can't use here wordpress native function wp_parse_url because this function fires early in advanced-cache.php
             */
            // phpcs:ignore WordPress.WP.AlternativeFunctions.parse_url_parse_url -- none available before WordPress is loaded.
            $parts = parse_url($this->original_url); //phpcs:ignore
            $this->scheme = isset($parts["scheme"]) ? strtolower($parts["scheme"]) : NULL;
            $this->port = isset($parts["port"]) ? $parts["port"] : NULL;
            $this->host = isset($parts["host"]) ? strtolower($parts["host"]) : NULL;
            $this->path = isset($parts["path"]) ? strtolower($parts["path"]) : "/";
            $this->query = isset($parts["query"]) ? $this->remove_tracking_params($parts["query"]) : NULL;
            $this->generate_path();
            $this->clear_url();
        }
        protected function get_data_from_request()
        {
            return $this->esc_url((isset($_SERVER['HTTP_X_FORWARDED_PROTO']) && $_SERVER['HTTP_X_FORWARDED_PROTO'] ? $_SERVER['HTTP_X_FORWARDED_PROTO'] : (isset($_SERVER['REQUEST_SCHEME']) && $_SERVER['REQUEST_SCHEME'] ? $_SERVER['REQUEST_SCHEME'] : (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] ? 'https' : 'http')) ).'://'.$_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI']);
        }
        public function get_path()
        {
            return $this->path;
        }
        public function get_url_path() {
            return strtolower($this->url_path);
        }
        private function generate_path() {
            $this->url_path = $this->host.(!empty($this->port) ? $this->port : '').(!empty($this->path) ? rtrim($this->path, '/') : '').'/'.(!empty($this->query) ? '_' : '').(!empty($this->query) ? preg_replace('/[^a-zA-Z0-9]/', '_', $this->query) . '/' : '');
        }
        public function get_url() {
            //temporary done dynamic url generation
            return $this->scheme . '://' . $this->host . (!empty($this->port) ? $this->port : '') . (!empty($this->path) ? rtrim($this->path, '/') : '') . '/' . (!empty($this->query) ? '?' . $this->query : '');
        }
        public function get_host() {
            return $this->host;
        }
        public function get_query() {
            return $this->query;
        }

        public function remove_tracking_params($query_params) {
            parse_str($query_params, $params_array);
            ksort($params_array);
            foreach($params_array as $key => $value) {
                if (in_array($key, $this->tracking_params)) {
                    unset($params_array[$key]);
                }
                if (in_array($key, $this->params_to_trim)) {
                    unset($params_array[$key]);
                }
            }
            return $this->http_build_query($params_array);
        }
        public function clear_url()
        {
            $url = preg_replace('/\?.*$/i', '', $this->original_url);
            $this->url = (rtrim($url, '/') . '/') . ($this->query ? '?' . $this->query : '');
        }
        public function add_query_param($name, $value = null) {
            if (empty($name)) {
                return false;
            }
            if (!empty($this->query)) {
                parse_str($this->query, $params_array);
            } else {
                $params_array = [];
            }
            if (!in_array($name, array_keys($params_array))) {
                $params_array[$name] = $value;
            }
            $this->query = $this->http_build_query($params_array);
        }

        public function http_build_query($input_array) {
            //making pairs
            $pairs = [];
            foreach ($input_array as $key => $value) {
                if (is_null($value) || empty($value)) {
                    $pairs[] = $key;
                } else if (is_array($value)) {
                    foreach ($value as $val) {
                        $pairs[] = $key . '[]=' . urlencode($val);
                    }
                } else {
                    $pairs[] = $key . '=' . urlencode($value);
                }
            }
            return implode('&', $pairs);
        }

        /*
         * function that escape url
         * based on wordpress default functions which are not accessible in advanced-cache.php
         * used only for secaping urls get from global $_SERVER
         */
        protected function esc_url($url) {
            $url = str_replace(' ', '%20', ltrim($url));
            $url = preg_replace('|[^a-z0-9-~+_.?#=!&;,/:%@$\|*\'()\[\]\\x80-\\xff]|i', '', $url);
            if ('' === $url) {
                return $url;
            }

            $strip = array('%0d', '%0a', '%0D', '%0A');
            $count = 1;
            while ($count) {
                $url = str_replace($strip, '', $url, $count);
            }

            $url = str_replace(';//', '://', $url);
            if (
                !$this->str_contains($url, ':') && !in_array($url[0], array('/', '#', '?'), true) &&
                !preg_match('/^[a-z0-9-]+?\.php/i', $url)
            ) {
                $url = 'http://' . $url;
            }
            if ($this->str_contains($url, '[') || $this->str_contains($url, ']')) {
                /*
                 * can't use here wordpress native function wp_parse_url because this function fires early in advanced-cache.php
                 */
                // phpcs:ignore WordPress.WP.AlternativeFunctions.parse_url_parse_url -- none available before WordPress is loaded.
                $parsed = parse_url($url); //phpcs:ignore
                $front = '';
                if (isset($parsed['scheme'])) {
                    $front .= $parsed['scheme'] . '://';
                } elseif ('/' === $url[0]) {
                    $front .= '//';
                }
                if (isset($parsed['user'])) {
                    $front .= $parsed['user'];
                }
                if (isset($parsed['pass'])) {
                    $front .= ':' . $parsed['pass'];
                }
                if (isset($parsed['user']) || isset($parsed['pass'])) {
                    $front .= '@';
                }
                if (isset($parsed['host'])) {
                    $front .= $parsed['host'];
                }
                if (isset($parsed['port'])) {
                    $front .= ':' . $parsed['port'];
                }
                $end_dirty = str_replace($front, '', $url);
                $end_clean = str_replace(array('[', ']'), array('%5B', '%5D'), $end_dirty);
                $url = str_replace($end_dirty, $end_clean, $url);
            }
            if ('/' === $url[0]) {
                $good_protocol_url = $url;
            } else {
                $allowed_protocols = array('http', 'https');
                $url = preg_replace('/[\x00-\x08\x0B\x0C\x0E-\x1F]/', '', $url);
                $url = preg_replace('/\\\\+0+/', '', $url);
                // Short-circuit if the string starts with `https://` or `http://`. Most common cases.
                if (
                    ($this->str_starts_with($url, 'https://') && in_array('https', $allowed_protocols, true)) ||
                    ($this->str_starts_with($url, 'http://') && in_array('http', $allowed_protocols, true))
                ) {
                    $good_protocol_url = $url;
                }
                if (strtolower($good_protocol_url) !== strtolower($url)) {
                    return '';
                }
            }
            return $good_protocol_url;
        }

        protected function str_contains($haystack, $needle) {
            if (function_exists('str_contains')) {
                return str_contains($haystack, $needle);
            }
            return strpos($haystack, $needle) !== false;
        }

        protected function str_starts_with($haystack, $needle) {
            if (function_exists('str_starts_with')) {
                return str_starts_with($haystack, $needle);
            }
            if ('' === $needle) {
                return true;
            }
            return 0 === strpos($haystack, $needle);
        }
    }
}